@@ -63,7 +63,10 @@ class ProxyConnection(Connection):
                     # only for one request
                     self.range_from = None
 
-                fn(method, url, body=body, headers=headers)
+                if 'body' in fn.func_code.co_varnames:
+                    fn(method, url, body=body, headers=headers)
+                else:  # swiftclient 2.0, ported to Requests
+                    fn(method, url, data=body, headers=headers)
             return request_x_forwarded_for
 
         parsed, conn = super(ProxyConnection, self).http_connection()
@@ -376,7 +379,7 @@ class ListDirCache(object):
     def key(self, index):
         """Returns a key for a user distributed cache."""
         logging.debug("cache key for %r" % [self.cffs.authurl, self.cffs.username, index])
-        if not hasattr(self, "_key_base") and self.cffs.username:
+        if not hasattr(self, "_key_base"):
             self._key_base = md5("%s%s" % (self.cffs.authurl, self.cffs.username)).hexdigest()
         return "%s-%s" % (self._key_base, md5(smart_str(index)).hexdigest())
 
@@ -401,6 +404,8 @@ class ListDirCache(object):
         if last_modified:
             if "." in last_modified:
                 last_modified, microseconds = last_modified.rsplit(".", 1)
+                if microseconds.endswith("Z"):
+                    microseconds = microseconds[:-1]
                 microseconds = float("0."+microseconds)
             else:
                 microseconds = 0.0
@@ -446,6 +451,9 @@ class ListDirCache(object):
             objects.extend(newobjects)
         logging.debug("total number of objects %s:" % len(objects))
 
+        if self.cffs.hide_part_dir:
+            manifests = []
+
         for obj in objects:
             # {u'bytes': 4820,  u'content_type': '...',  u'hash': u'...',  u'last_modified': u'2008-11-05T00:56:00.406565',  u'name': u'new_object'},
             if 'subdir' in obj:
@@ -457,6 +465,8 @@ class ListDirCache(object):
                 manifest_obj = self.conn.head_object(container, obj['name'])
                 logging.debug("possible manifest file: %r" % manifest_obj)
                 if 'x-object-manifest' in manifest_obj:
+                    if self.cffs.hide_part_dir:
+                        manifests.append(manifest_obj['x-object-manifest'])
                     logging.debug("manifest found: %s" % manifest_obj['x-object-manifest'])
                     obj['hash'] = manifest_obj['etag']
                     obj['bytes'] = int(manifest_obj['content-length'])
@@ -465,6 +475,15 @@ class ListDirCache(object):
             name = posixpath.basename(obj['name']).encode("utf-8")
             cache[name] = self._make_stat(**obj)
 
+        if self.cffs.hide_part_dir:
+            for manifest in manifests:
+                manifest_container, manifest_obj = parse_fspath('/' + manifest)
+                if manifest_container == container:
+                    for cache_obj in cache.copy():
+                        if manifest_obj == os.path.join(path, cache_obj):
+                            logging.debug("hidding part dir %r" % manifest_obj)
+                            del cache[cache_obj]
+
     def listdir_root(self, cache):
         """Fills cache with the list of containers"""
         logging.debug("listdir root")
@@ -587,7 +606,7 @@ class ObjectStorageFS(object):
     memcache_hosts = None
 
     @translate_objectstorage_error
-    def __init__(self, username, api_key, authurl, keystone=None):
+    def __init__(self, username, api_key, authurl, keystone=None, hide_part_dir=False):
         """
         Create the Object Storage connection.
 
@@ -599,6 +618,7 @@ class ObjectStorageFS(object):
         self.conn = None
         self.authurl = authurl
         self.keystone = keystone
+        self.hide_part_dir = hide_part_dir
         # A cache to hold the information from the last listdir
         self._listdir_cache = ListDirCache(self)
         self._cwd = '/'
@@ -794,10 +814,19 @@ class ObjectStorageFS(object):
         if self.isdir(path):
             raise IOSError(EACCES, "Can't remove a directory (use rmdir instead)")
 
+        meta = self.conn.head_object(container, name)
+        if 'x-object-manifest' in meta:
+            self._remove_path_folder_files('/' + meta['x-object-manifest'])
         self.conn.delete_object(container, name)
         self._listdir_cache.flush(posixpath.dirname(path))
         return not name
 
+    def _remove_path_folder_files(self, path):
+        logging.info("Removing manifest file's parts from:  %s" % path)
+        files = self.listdir(path)
+        for file in files:
+          self.remove(path + '/' + file)
+
     @translate_objectstorage_error
     def _rename_container(self, src_container_name, dst_container_name):
         """Rename src_container_name into dst_container_name"""
@@ -851,12 +880,17 @@ class ObjectStorageFS(object):
         if not self.isdir(posixpath.split(dst)[0]):
             raise IOSError(ENOENT, "Can't copy %r to %r, destination directory doesn't exist" % (src, dst))
 
-        # check src/dst containers
-        self._container_exists(src_container_name)
+        # check dst container
         self._container_exists(dst_container_name)
 
         # Do the rename of the file/dir
-        headers = { 'x-copy-from': "/%s/%s" % (src_container_name, src_path) }
+        meta = self.conn.head_object(src_container_name, src_path)
+        if 'x-object-manifest' in meta:
+            # a manifest file
+            headers = { 'x-object-manifest': meta['x-object-manifest'] }
+        else:
+            # regular file
+            headers = { 'x-copy-from': "/%s/%s" % (src_container_name, src_path) }
         self.conn.put_object(dst_container_name, dst_path, headers=headers, contents=None)
         # Delete src
         self.conn.delete_object(src_container_name, src_path)
@@ -1008,4 +1042,3 @@ class ObjectStorageFS(object):
 
         meta = self.conn.head_object(container, name)
         return meta["etag"]
-
